<?php
/**
 * Plugin Outdent: Removes one level of indenting.
 *
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author     Christopher Smith <chris@jalakai.co.uk>
 * @author     Vadim Nevorotin <malamut@ubuntu.ru>
 */

// DokuWiki essentials
if (!defined('DOKU_INC')) die();
if (!defined('DOKU_PLUGIN'))
    define('DOKU_PLUGIN', DOKU_INC . 'lib/plugins/');
require_once (DOKU_PLUGIN . 'syntax.php');

class syntax_plugin_outdent extends DokuWiki_Syntax_Plugin {

    function getType() {
        return 'baseonly';
    }
    function getSort() {
        return 50;            // same as header
    }                       

    function connectTo($mode) {
        $this->Lexer->addSpecialPattern('\n[ \t]*(?:'.$this->getConf('syntax').')+[ \t]*(?=\n)',$mode,'plugin_outdent');
    }

    function handle($match, $state, $pos, &$handler) {
      if ($state == DOKU_LEXER_SPECIAL) {
        $outdent = (int)(strlen(trim($match)) / strlen($this->getConf('syntax')));
        $level = $this->_getLevel($handler->calls);
        if ($level > 1) {
          if ($level > $outdent) {
            $handler->_addCall('section_close', array(), $pos);
            $handler->_addCall('section_open', array($level-$outdent), $pos);
          } else {
            $handler->_addCall('section_close', array(), $pos);
            $handler->_addCall('section_open', array(1), $pos);
          }
        }
      }
      return false;
    }

    function render($mode, &$renderer, $data) {
      return false;
    }

    function _getLevel(&$calls) {
      for ($i=count($calls); $i >= 0; $i--) {
          if ($calls[$i][0] == 'header') return $calls[$i][1][1];
        if ($calls[$i][0] == 'section_open') return $calls[$i][1][0];
      }
      return 0;
    }
}
